/**
A Backtracking program to solve the Sudoku puzzle
**/

#include<iostream>
using namespace std;

// UNASSIGNED is used for empty cells in grid
const int UNASSIGNED = 0;

// N is used for size of Sudoku grid. Size is NxN
const int N = 9;

// Finds an entry in grid that is still unassigned, if any:
bool FindUnassignedLocation(int grid[N][N], int &row, int &col);

// Checks whether it is legal to assign num to the given row,col
bool isSafe(int grid[N][N], int row, int col, int num);

/* Takes a partially filled-in grid and attempts to assign values to
  all unassigned locations in such a way as to meet the requirements
  for Sudoku: non-duplication across rows, columns, and zones */
bool SolveSudoku(int grid[N][N], string indent = "")
{
    int row, col;

    // If there is no unassigned location, we are done
    // Otherwise, row and col are modified
    // (since they are passed by ref)
    if(!FindUnassignedLocation(grid, row, col))
       return true; // Total success!

    // consider digits 1 to 9
    for(int num = 1; num <= 9; num++)
    {
        // Does this digit produce any immediate conflicts?
        if(isSafe(grid, row, col, num))
        {
            // make tentative assignment
            grid[row][col] = num;
            cout << indent << "grid[" << row << "]" << "[" << col << "] = " << num << endl;

            // return, if success, yay!
            if(SolveSudoku(grid, indent + " "))
                return true; // like it occurs after recursive call

            // failure, un-do (backtrack) and try again
            grid[row][col] = UNASSIGNED;
        }
    }
    cout << indent << "grid[" << row << "]" << "[" << col << "] = " << "1-9 all conflict!" << endl;
    return false; // this triggers backtracking
}

/* Searches the grid to find an entry that is still unassigned. If
   found, the reference parameters row, col will be end up set
   (by the for loop) to the location that is unassigned,
   and true is returned. If no unassigned entries remain,
   false is returned. */
bool FindUnassignedLocation(int grid[N][N], int &row, int &col)
{
    for(row = 0; row < N; row++)
        for(col = 0; col < N; col++)
            if(grid[row][col] == UNASSIGNED)
                return true; // What happens to rol and col here?
    return false;
}

/* Returns a boolean which indicates whether any assigned entry
   in the specified row matches the given number. */
bool UsedInRow(int grid[N][N], int row, int num)
{
    for(int col = 0; col < N; col++)
        if(grid[row][col] == num)
            return true;
    return false;
}

/* Returns a boolean which indicates whether any assigned entry
   in the specified column matches the given number. */
bool UsedInCol(int grid[N][N], int col, int num)
{
    for(int row = 0; row < N; row++)
        if(grid[row][col] == num)
            return true;
    return false;
}

/* Returns a boolean which indicates whether any assigned entry
   within the specified 3x3 box matches the given number. */
bool UsedInBox(int grid[N][N], int boxStartRow, int boxStartCol, int num)
{
    for(int row = 0; row < 3; row++)
        for(int col = 0; col < 3; col++)
            if(grid[row+boxStartRow][col+boxStartCol] == num)
                return true;
    return false;
}

/* Returns a boolean which indicates whether it will be legal to assign
   num to the given row,col location. */
bool isSafe(int grid[N][N], int row, int col, int num)
{
    /* Check if 'num' is not already placed in current row,
       current column and current 3x3 box */
    return !UsedInRow(grid, row, num) &&
           !UsedInCol(grid, col, num) &&
           !UsedInBox(grid, row - row % 3 , col - col % 3, num);
}

/* Print grid  */
void printGrid(int grid[N][N])
{
    for(int row = 0; row < N; row++)
    {
        for(int col = 0; col < N; col++)
            cout << grid[row][col] << " ";
        cout << endl;
    }
    cout << endl;
}


int main()
{
    // 0 for unassigned cells
    int grid[N][N] = {{3, 0, 6, 5, 0, 8, 4, 0, 0},
                      {5, 2, 0, 0, 0, 0, 0, 0, 0},
                      {0, 8, 7, 0, 0, 0, 0, 3, 1},
                      {0, 0, 3, 0, 1, 0, 0, 8, 0},
                      {9, 0, 0, 8, 6, 3, 0, 0, 5},
                      {0, 5, 0, 0, 9, 0, 6, 0, 0},
                      {1, 3, 0, 0, 0, 0, 2, 5, 0},
                      {0, 0, 0, 0, 0, 0, 0, 7, 4},
                      {0, 0, 5, 2, 0, 6, 3, 0, 0}};

    cout << "Problem: " << endl;

    printGrid(grid);

    cout << "Solution: " << endl;

    if(SolveSudoku(grid) == true)
        printGrid(grid);
    else
        cout << "No solution exists";

    return 0;
}
